'''
Created on Apr 24, 2014

@author: pascale
'''
import unittest
import rttov
import h5py
import rmodel
import logging
import rttovgui_unittest_class
class Test(rttovgui_unittest_class.RttovGuiUnitTest):


    def setUp(self):
       
        self.p=rmodel.project.Project()        
        
        level_logging=logging.DEBUG        
        logging.basicConfig(filename=self.p.config.ENV['GUI_WRK_DIR']+"/rttovgui_test_clouds.log" ,
                            format="[%(asctime)s] %(levelname)s [%(module)s:%(funcName)s:%(lineno)d] %(message)s",
                            level=level_logging,
                            datefmt="%Y:%m:%d %H:%M:%S",
                            filemode="w")       
        logging.info("start test cloud")  
        
  
        profileName= self.p.config.ENV["RTTOV_GUI_PREFIX"]+"/rttov_tests/cldaer101lev_allgas.H5" 
        err=self.p.openProfile(profileName)
        self.assertEqual(err,0)
  

    def tearDown(self):
        pass


    def test_read_profile(self):
        profileName=self.p.config.ENV["RTTOV_GUI_PROFILE_DIR"]+"/standard54lev_allgas.H5"
        err=self.p.openProfile(profileName)
        self.assertEqual(err, 0) 
        self.assertGreater(self.p.myProfile['BE'],  0.20)

        
    def test_wv_clw(self) :
        print "------------------test CLW-------------------------------"
        profileName=self.p.config.ENV["RTTOV_GUI_PROFILE_DIR"]+"/standard101lev_clw.H5"
        self.p.openProfile(profileName)
        coefFile = self.p.config.ENV['RTTOV_GUI_COEFF_DIR'] + "/rttov7pred54L/rtcoef_dmsp_18_ssmis.dat" 
        self.p.myCoeffs.fileName["standard"] = coefFile
        err=self.p.loadCoefficients() 
        self.assertEqual(err, 0) 
        self.p.ctrlCoherence()
        self.assertTrue(self.p.myOption.statusOption["CLW_DATA"])
        profileName=self.p.config.ENV["RTTOV_GUI_PROFILE_DIR"]+"/cld101lev_allgas.H5"
        self.p.openProfile(profileName)
        self.p.ctrlCoherence()
        self.assertFalse(self.p.myOption.statusOption["CLW_DATA"])
        self.assertFalse(self.p.myOption["CLW_DATA"])
        
    
    def test_clouds(self):
        import glob
        import re
        import os
        print "-------------------TEST CLOUD-1----------------------"

        profile_with=self.p.config.ENV["RTTOV_GUI_PROFILE_DIR"]+"/cldaer101lev_allgas.H5"
        profile_without=self.p.config.ENV["RTTOV_GUI_PROFILE_DIR"]+"/standard101lev_allgas.H5"
        
       
        
        self.p.ctrlCoherence()
        list_cld=[]
        for coefFile in  glob.glob(self.p.config.ENV["RTTOV_GUI_COEFF_DIR"]+"/cldaer/sccld*.dat") :
            print "add ",coefFile
            list_cld.append(coefFile)
        

        cas=0
        for coefFile  in list_cld :  
            cas=cas+1
            print "############################ TEST CLOUDS 1 cas",cas,"############"
            self.p.myCoeffs.fileName["clouds"]=coefFile
            std=re.sub('sccld','rt', re.sub("cldaer","rttov7pred54L",coefFile))
            print "file ",std
            if os.path.exists(std) :
                print "this file exists"
                self.p.myCoeffs.fileName["standard"]=std  

                print  ">>>>>>>>>>>>>>>>>>>>>>>test clouds 1 with" ,  self.p.myCoeffs.fileName
                err=self.p.loadCoefficients()      
                self.assertEqual(err,0)
                if err != 0 :
                    print" cannot load coeff", self.p.myCoeffs.fileName
                    import sys
                    sys.exit(1)
                print ">>>>>>>>>>>>>>>>>>>>>coefFile;",self.p.myCoeffs.fileName ," loaded"
                print ">>>>>>>>>>>test with clouds ",profile_with
                err=self.p.openProfile(profile_with, 1)
                self.assertEqual(err,0)
                
                self.p.ctrlCoherence()
                self.check_option(self.p)
                err=self.p.runDirect()
                self.assertEqual(err,0)
                err=self.p.runK()
                self.assertEqual(err,0)
                self.check_option(self.p)
                self.assertTrue(self.p.myOption.statusOption["ADDCLOUDS"])
                if self.p.myOption.statusOption["ADDCLOUDS"] != True : print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!PB ICI"
                self.p.myOption["ADDCLOUDS"]=True
                err=self.p.runDirect()
                self.assertEqual(err,0)
                err=self.p.runK()
                self.assertEqual(err,0)
                self.check_option(self.p)
                self.assertTrue(self.p.myOption.statusOption["ADDCLOUDS"])
                self.assertTrue(self.p.myOption["ADDCLOUDS"])
                self.p.myProfile.removeCloud('STCO')
                self.p.myProfile.removeCloud('STMA')
                self.p.myProfile.removeCloud('CUCC')
                self.p.myProfile.removeCloud('CUCP')
                self.p.myProfile.removeCloud('CUMA')
                self.p.myProfile.removeCloud('CIRR')
                
                print ">>>>>>>>>remove clouds and run"
                self.p.ctrlCoherence()
                self.check_option(self.p)
                err=self.p.runDirect()
                self.assertEqual(err,0)
                print ">>>>>>>>>>>test without ",profile_with
                err=self.p.openProfile(profile_without, 1)
                self.assertEqual(err,0)
                
    
                self.p.ctrlCoherence()
                self.check_option(self.p)
             
                self.assertFalse(self.p.myOption.statusOption["ADDCLOUDS"])
                self.assertFalse(self.p.myOption["ADDCLOUDS"])
            
                print ">>>>>>>>>>>>>>>>>>>>>>OK for ",coefFile, profile_with, "cas",cas
        
                    
        
        print ">>>>>>>>>>>>>>>>> end test_aerosols" 
        
        
    def test_pc_cloud1(self):
        print "-------------------PC CLOUD-1----------------------"
       
        
        self.p.myOption["ADDPC"]=True
        coefFile = self.p.config.ENV['RTTOV_GUI_COEFF_DIR'] + "/rttov9pred101L/rtcoef_metop_2_iasi.H5"
        pcFile=self.p.config.ENV['RTTOV_GUI_COEFF_DIR'] + "/pc/pccoef_metop_2_iasi.H5"
        
        #pcCloudsFile = sxelf.p.config.ENV['RTTOV_GUI_COEFF_DIR'] + "/pc/pccoefcld_metop_2_iasi.dat" 
        self.p.myCoeffs.fileName["standard"] = coefFile
        self.p.myCoeffs.fileName["PC"] = pcFile
        self.p.myProfile['SKIN']['SURFTYPE']=1
        #self.p.myCoeffs.fileName["PC"] =pcCloudsFile
        
        #self.p.myCoeffs.fileName["clouds"] =pcCloudsFile
        print self.p.myCoeffs.fileName
        self.p.myOption.display()
        err=self.p.loadCoefficients() 
        self.assertEqual(err, 0)  
        self.p.myProfile['SKIN']['SURFTYPE'] = 1
        print "is cloud ?",self.p.myCoeffs.isPCClouds()
        
        err=self.p.runPC()    
        self.assertEqual(err, 0)  

        print "ADDCLOUD ?:", self.p.myOption["ADDCLOUDS"]
        self.p.myOption["ADDCLOUDS"]=True
        err=self.p.runPC()    
        self.assertEqual(err, 0)
        self.check_option(self.p)
        if self.p.myCoeffs.isPCClouds() :
            self.assertTrue(self.p.myOption["ADDCLOUDS"])
            self.assertTrue(self.p.myOption.statusOption["ADDCLOUDS"])
        else :
            self.assertFalse(self.p.myOption["ADDCLOUDS"])   
            self.assertFalse(self.p.myOption.statusOption["ADDCLOUDS"])
        print ">>>>>>>>>>>>> ADDCLOUD ?:", self.p.myOption["ADDCLOUDS"]
        err=self.p.runPCK()
        self.assertEqual(err, 0)
        
    def test_pc_cloud2(self):  
        print ">>>>>>>>>>>test_pc_cloud2"
        profileName=self.p.config.ENV["RTTOV_GUI_PROFILE_DIR"]+"/standard54lev_allgas.H5"
        self.p.openProfile(profileName)

        print "profile", profileName," has cloud ?",self.p.myProfile.hasClouds() 
        self.assertFalse(self.p.myProfile.hasClouds())   
           
        profileName= self.p.config.ENV["RTTOV_GUI_PREFIX"]+"/rttov_tests/cldaer101lev_allgas.H5" 
        self.p.openProfile(profileName)
        
        print "profile", profileName," has cloud ?",self.p.myProfile.hasClouds() 
        self.assertTrue(self.p.myProfile.hasClouds())   
        file= "../rttov_tests/cldaer101lev_allgas.H5"

    def test_pc_cloud3(self): 
        print "-------------------PC CLOUD-3------RTTOV CLOUDS ----------------"
        profileName= self.p.config.ENV["RTTOV_GUI_PREFIX"]+"/rttov_tests/cldaer101lev_allgas.H5" 
        #profileName= self.p.config.ENV["RTTOV_GUI_PROFILE_DIR"]+"/cld101lev_allgas.H5" 
        err=self.p.openProfile(profileName)
        self.assertEqual(err,0)
        self.p.myOption["ADDPC"]=True
        coefFile = self.p.config.ENV['RTTOV_GUI_COEFF_DIR'] + "/rttov9pred101L_old/rtcoef_metop_2_iasi.H5"
        pcFile=self.p.config.ENV['RTTOV_GUI_COEFF_DIR'] + "/pc//pccoefcld_metop_2_iasi.dat"
        
        pcCloudsFile = self.p.config.ENV['RTTOV_GUI_COEFF_DIR'] + "/cldaer/sccldcoef_metop_2_iasi.H5" 
        self.p.myCoeffs.fileName["standard"] = coefFile
        self.p.myCoeffs.fileName["PC"] = pcFile
        self.p.myCoeffs.fileName["clouds"] =pcCloudsFile
        self.p.myProfile['SKIN']['SURFTYPE']=1
        

        print self.p.myCoeffs.fileName
        #self.p.myOption.display()
        err=self.p.loadCoefficients(redirectOutput=False) 
        print "load coefficient return code" ,err
       
        self.assertEqual(err, 0)  
        self.p.myProfile['SKIN']['SURFTYPE'] = 1
        print "is pc cloud ?",self.p.myCoeffs.isPCClouds()
        self.assertTrue(self.p.myCoeffs.isPCClouds())
        print "coeff has cloud ?",self.p.myCoeffs.hasClouds()
        self.assertTrue(self.p.myCoeffs.hasClouds())
        self.p.ctrlCoherence()
        print "profile has cloud ?",self.p.myProfile.hasClouds() 
        self.assertTrue(self.p.myProfile.hasClouds())
        print "################################################### launch the first run without clouds : "
        err=self.p.runPC()    
        self.assertEqual(err, 0)  
        self.assertTrue(self.p.myOption.statusOption["ADDCLOUDS"])
        
        print ">>>>>>>>>>>>>>>>>>>ADDCLOUD ?:", self.p.myOption["ADDCLOUDS"]

        self.p.myOption["ADDCLOUDS"]=True
        self.p.ctrlCoherence()
        print ">>>>>>>>>>>>>>>>>>>ADDCLOUDS ?:", self.p.myOption["ADDCLOUDS"]
        print ">>>>>>>>>>>>>>>>>>>>>>ADDPC ?:", self.p.myOption["ADDPC"]
        print "##################################################### launch the second run with clouds : "
        err=self.p.runPC()        
        self.check_option(self.p)
        self.assertTrue(self.p.myOption["ADDCLOUDS"])   
        print ">>>>>>>>>>>>>>>>>>>ADDCLOUDS ?:", self.p.myOption["ADDCLOUDS"]
        self.assertEqual(err, 0)
        
        if self.p.myCoeffs.isPCClouds() :
            self.assertTrue(self.p.myOption["ADDCLOUDS"])
            self.assertTrue(self.p.myOption.statusOption["ADDCLOUDS"])
        else :
            self.assertFalse(self.p.myOption["ADDCLOUDS"])   
            self.assertFalse(self.p.myOption.statusOption["ADDCLOUDS"])
        print ">>>>>>>>>>>>> ADDCLOUD ?:", self.p.myOption["ADDCLOUDS"]
        err=self.p.runPCK()
        self.assertEqual(err, 0)      
        self.assertTrue(self.p.myOption["ADDCLOUDS"])    
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.test_read_profile']
    unittest.main()
