#!/bin/sh
# Script to run the EXAMPLE_PC_FWD code
#
# The result is compared to a reference file. See users guide
# to note what are the "normal" differences.
#


# Set BIN directory if supplied
BIN=`perl -e 'for(@ARGV){m/^BIN=(\S+)$/o && print "$1";}' $*`
if [ "x$BIN" = "x" ]
then
  BIN=bin
fi

######## Edit this section for your pathnames and test case input ######

# Path relative to the rttov_test directory:
TEST=./test_example_pc_fwd.1

# Paths relative to the rttov_test/${TEST} directory:
BINDIR=../../$BIN                           # BIN directory (may be set with BIN= argument)
REF_TEST=../test_example_pc_fwd.2           # Test reference data
DATART=../../rtcoef_rttov11/rttov9pred101L  # Coefficients directory (rtcoef)
DATAPC=../../rtcoef_rttov11/pc              # Coefficients directory (pccoef)

# Test case input data
COEF_FILENAME="rtcoef_metop_2_iasi.dat"
PCCOEF_FILENAME="pccoef_metop_2_iasi.dat"
PROF_FILENAME="prof.dat"
RADREC_FILENAME="radrec.dat" # Reconstructed channel list, this file need not exist (see example_pc_fwd.F90)
NPROF=1         # Number of profiles defined in prof.dat
NLEVELS=51      # Number of profile levels
IPCBND=1        # PC band - AIRS: 1; IASI: 1-3 (see user guide)
IPCREG=1        # PC regression set - 1-4
NPCSCORES=400   # Number of PC scores to calculate (1-400)

########################################################################

ARG_ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
if [ ! "x$ARG_ARCH" = "x" ]; then
  ARCH=$ARG_ARCH
fi
if [ "x$ARCH" = "x" ];
then
  echo 'Please supply ARCH'
  exit 1
fi

CWD=`pwd`
cd $TEST

echo " "
echo " "
echo " Test PC forward "
echo " "

echo  "Coef filename:       ${COEF_FILENAME}"
echo  "PC coef filename:    ${PCCOEF_FILENAME}"
echo  "Input profile file:  ${PROF_FILENAME}"
echo  "Rad. rec. file:      ${RADREC_FILENAME}"
echo  "Number of profiles:  ${NPROF}"
echo  "Number of levels:    ${NLEVELS}"
echo  "PC band:             ${IPCBND}"
echo  "PC regression set:   ${IPCREG}"
echo  "Number of PC scores: ${NPCSCORES}"

# Coefficient files
rm -f $COEF_FILENAME
if [ -s $DATART/$COEF_FILENAME ]; then
  ln -s $DATART/$COEF_FILENAME
else
  echo "Coef file $DATART/$COEF_FILENAME not found, aborting..."
  exit 1
fi

rm -f $PCCOEF_FILENAME
if [ -s $DATAPC/$PCCOEF_FILENAME ]; then
  ln -s $DATAPC/$PCCOEF_FILENAME
else
  echo "PC coef file $DATAPC/$PCCOEF_FILENAME not found, aborting..."
  exit 1
fi


$BINDIR/example_pc_fwd.exe << EOF
${COEF_FILENAME}  , Coefficient filename
${PCCOEF_FILENAME}, PC coefficient filename
${PROF_FILENAME}  , Input profile filename
${RADREC_FILENAME}, Rad. rec. filename
${NPROF}          , Number of profiles
${NLEVELS}        , Number of levels
${IPCBND}         , PC band number
${IPCREG}         , PC regression set
${NPCSCORES}      , Number of PC scores
EOF

if [ $? -ne 0 ]; then
  echo " "
  echo "TEST FAILED"
  echo " "
  exit 1
fi

OUT_FILE=output_example_pc_fwd.dat
DIFF_FILE=diff_example_pc_fwd.${ARCH}

mv ${OUT_FILE} ${OUT_FILE}.${ARCH}

if [ $? -ne 0 ]; then
  echo "Expected output file not found"
  exit 1
fi

echo
echo "Output is in the file ${TEST}/${OUT_FILE}.${ARCH}"

if [ -f ${REF_TEST}/${OUT_FILE} ]; then

  diff -biw ${OUT_FILE}.${ARCH} ${REF_TEST}/${OUT_FILE} > $DIFF_FILE

  if [ `stat -c %s $DIFF_FILE` -eq 0 ]; then
    echo " "
    echo "Diff file has zero size: TEST SUCCESSFUL"
    echo " "
  else
    echo "--- Diff file contents: ---"
    cat $DIFF_FILE
    echo "---------------------------"
  fi
else
  echo "Test reference output not found"
fi
echo

rm -f $COEF_FILENAME $PCCOEF_FILENAME

cd $CWD

exit
